;  Teeny little x86 PC bootsector program 
;  Designed for the Netwide assembler, "nasm".
;  Compile into a boot block with "nasm -f bin boot.s -o boot.bin"
;   See http://nasm.sourceforge.net/doc/html/nasmdoc0.html
;  Orion Sky Lawlor, olawlor@acm.org, 2005/12/20 (Public Domain)

BITS 16         ; Starts running in 16-bit segmented DOS mode
SECTION .text   ; Put everything into the code ("text") segment
GLOBAL _start   ; Tells linker where code starts

_start: ; BIOS loads this code, and starts running right here

	mov ebp,esp; Set up base pointer
	sub esp,32; Make room for local variables
	mov edi,esp; Start of x array
	sub esp,25; Make room for array of x columns
	sub esp,80; Make room for big output buffer
	
	mov long [ebp-4],22 ; User's row
	mov long [ebp-8],40 ; User's column
	
redraw:
	; Move cursor to screen top-right
	mov ah,0x02
	mov bh,0x00
	mov dx,0
	int 0x10
	
; Loop over screen:
	mov edx,0 ; Line counter (increases)
drawLine:
;   Fill line with spaces
	mov ecx,0 ; Column counter (increases)
spaces:
	mov byte [esp+ecx],' '
	inc cx
	cmp cx,80
	jl spaces
	
;   Draw in everything onscreen
	cmp edx,[ebp-4]
	jne skip_user
	mov eax,[ebp-8] ; user column
	mov byte [esp+eax],1 ; Draw in user in his column
skip_user:
	
	
;   Copy line to screen
	mov bx, 0x0007  ; select output color
	mov ah,0x0e
	mov cx,0 ; Column counter (increases)
onscreen:
	mov al,[esp+ecx]
	int 0x10
	inc cx
	cmp cx,80
	jl onscreen
	
	inc dx
	cmp dx,24
	jl drawLine

; Slow down the action to a playable rate
	mov eax,0x100000  ; STOOPID busy-wait loop.  FIXME: figure out how to pause
stoopid: dec eax
	jne stoopid
	
	
; Check for a keypress
	mov ah,0x01
	int 0x16
	jz skip_keystroke
	; Got a keystroke
	mov ah,0x00 ; Grab the keystroke
	int 0x16
	mov bx,0
	cmp al,'a'
	je moveLeft
	cmp al,'d'
	je moveRight
	jmp skip_keystroke ; Uninteresting key
	
moveLeft:
	dec long [ebp-8]
	jmp checkPos
moveRight:
	inc long [ebp-8]
	jmp checkPos

checkPos:
	mov eax,[ebp-8]
	jl badPos
	cmp eax,80
	jge badPos
	jmp skip_keystroke
badPos:
	mov long [ebp-8],40
	
skip_keystroke:
	
	jmp redraw

reboot:
	jmp  0xf000:0xfff0 ; Jump to BIOS reset code

; The boot sector's last two bytes must contain this magic flag.
; This tells the BIOS it's OK to boot here; without this, 
;  you'll get the BIOS "Operating system not found" error.
	times 510-($-_start) db 0  ; pad to offset 510 
	dw 0xAA55 ; Magic value needed by BIOS
END
