;  Teeny little x86 PC bootsector program 
;  Designed for the Netwide assembler, "nasm".
;  Compile into a boot block with "nasm -f bin boot.s -o boot.bin"
;   See http://nasm.sourceforge.net/doc/html/nasmdoc0.html
;  Orion Sky Lawlor, olawlor@acm.org, 2005/12/20 (Public Domain)

BITS 16         ; Starts running in 16-bit segmented DOS mode
SECTION .text   ; Put everything into the code ("text") segment
GLOBAL _start   ; Tells linker where code starts

_start: ; BIOS loads this code, and starts running right here
	mov bx, 0x0007  ; select output color
	mov ax, 0x0e42  ; 'B'
	int 0x10  ; "output character" interrupt
	mov ax, 0x0e3E  ; '>'
	int 0x10  ; "output character" interrupt

waitForKey:
	mov ax, 0x0000
	int 0x16  ; Wait for a keypress.  Key returned in register al.
	
	cmp al,'r'
	je reboot
	
	mov ah, 0x0e
	int 0x10
	
	jmp waitForKey

reboot:
	jmp  0xf000:0xfff0 ; Jump to BIOS reset code

; The boot sector's last two bytes must contain this magic flag.
; This tells the BIOS it's OK to boot here; without this, 
;  you'll get the BIOS "Operating system not found" error.
	times 510-($-_start) db 0  ; pad to offset 510 
	dw 0xAA55 ; Magic value needed by BIOS
END
